@@ -84,3 +84,18 @@ def get_func_by_name(fn_name):
         module = getattr(module, clazz)
 
     return getattr(module, fn)
+
+
+class fake_tqdm:
+
+    def __init__(self):
+        pass
+
+    def update(self, index: int):
+        return self
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return self
