@@ -1,10 +1,11 @@
+import contextlib
 import glob
 from pathlib import Path
 from typing import Optional, List, Callable, Any
 
 from tqdm.auto import tqdm
 
-from shmr.misc import get_filepath_template, get_func_by_name
+from shmr.misc import get_filepath_template, get_func_by_name, fake_tqdm
 from shmr.partition import Partition
 from shmr.partition_writer import PartitionWriter
 
@@ -53,23 +54,23 @@ class ListPartition:
             # TODO: cal the records_per_partition based on num_partitions
 
         part_counter = 0
-        start = 0
 
         writer = None
-        try:
-            writer = PartitionWriter(outfile % part_counter).open()
-            for inpart in self.partitions:
-                with inpart._open() as f:
-                    for i, line in (tqdm(enumerate(f), initial=start, total=self.size) if verbose else enumerate(f)):
-                        if (i + 1) % records_per_partition == 0:
-                            writer.close()
-                            part_counter += 1
-                            writer = PartitionWriter(outfile % part_counter).open()
-                        writer.write(line)
-                        start += 1
-        finally:
-            if writer is not None:
-                writer.close()
+        with (tqdm(total=self.size) if verbose else fake_tqdm()) as pbar:
+            try:
+                writer = PartitionWriter(outfile % part_counter).open()
+                for inpart in self.partitions:
+                    with inpart._open() as f:
+                        for i, line in enumerate(f):
+                            if (i + 1) % records_per_partition == 0:
+                                writer.close()
+                                part_counter += 1
+                                writer = PartitionWriter(outfile % part_counter).open()
+                            writer.write(line)
+                            pbar.update(1)
+            finally:
+                if writer is not None:
+                    writer.close()
 
     def concat(self, outfile: str, verbose: bool = True):
         """Concatenate partitions to one file
@@ -81,13 +82,12 @@ class ListPartition:
         Returns:
             ValueError: if the output directory does not exist
         """
-        with PartitionWriter(outfile) as g:
-            start = 0
+        with PartitionWriter(outfile) as g, (tqdm(total=self.size) if verbose else fake_tqdm()) as pbar:
             for inpart in self.partitions:
                 with inpart._open() as f:
-                    for line in (tqdm(f, initial=start, total=self.size) if verbose else f):
+                    for line in f:
                         g.write(line)
-                        start += 1
+                        pbar.update(1)
 
     def reduce(self, fn: str, outfile: str, init_val: Any = None, verbose: bool = True):
         """Reduce
@@ -104,24 +104,21 @@ class ListPartition:
         else:
             accum = None
 
-        with PartitionWriter(outfile) as g:
-            start = 0
-            total = self._size()
-
+        with PartitionWriter(outfile) as g, (tqdm(total=self.size) if verbose else fake_tqdm()) as pbar:
             for inpart in self.partitions:
                 with inpart._open() as f:
                     if accum is None:
                         try:
                             record = inpart.deser_fn(next(f))
                             accum = fn(record)
-                            start += 1
+                            if verbose:
+                                pbar.update(1)
                         except StopIteration:
                             pass
 
-                    for line in tqdm(f, initial=start, total=total) if verbose else f:
+                    for line in f:
                         record = inpart.deser_fn(line)
                         accum = fn(record, accum)
-                        start += 1
-
+                        pbar.update(1)
             g.write(self.ser_fn(accum))
             g.write_new_line()
